import { reactive, watch } from "vue";
import { useQuery, useMutation } from "vue-query";
import { defu } from "defu";
import { indexRequest, getRequest, createRequest, updateRequest, deleteRequest } from "./requests.mjs";
import { processIndexData, processGetData } from "./utils.mjs";
const getNullKeys = (element) => Object.entries(element ?? {}).filter((entry) => entry[1] === null).map((entry) => entry[0]);
const removeNullKeys = (receivedObject, currentObject) => {
  const nullAttributeKeys = getNullKeys(receivedObject?.attributes);
  const nullRelationshipKeys = getNullKeys(receivedObject?.relationships);
  if (!currentObject)
    return currentObject;
  const attributes = !!currentObject?.attributes ? {} : { ...currentObject.attributes };
  const relationships = !!currentObject?.relationships ? {} : { ...currentObject.relationships };
  nullAttributeKeys.forEach((key) => delete attributes[key]);
  nullRelationshipKeys.forEach((key) => delete relationships[key]);
  return { ...currentObject, attributes, relationships };
};
export const storeVueQuery = (store, apiResourceType, queryOptions) => {
  const resourceType = apiResourceType || store.$id;
  const indexKey = reactive([resourceType, {}]);
  const indexQuery = useQuery(
    indexKey,
    () => indexRequest(resourceType, indexKey[1]),
    {
      keepPreviousData: true,
      staleTime: 1e4,
      ...queryOptions?.index
    }
  );
  const index = async (queryParams = {}) => {
    indexKey[1] = queryParams;
    const data = indexQuery.data.value;
    if (data) {
      processIndexData(data, store);
    }
  };
  watch(indexQuery.data, (newData) => {
    const data = newData;
    if (data) {
      processIndexData(data, store);
    }
  });
  const getKey = reactive([resourceType, "", {}]);
  const getQuery = useQuery(
    getKey,
    () => getRequest(resourceType, getKey[1], getKey[2]),
    {
      staleTime: 1e4,
      ...queryOptions?.get
    }
  );
  const get = async (id, queryParams = {}) => {
    getKey[1] = id;
    getKey[2] = queryParams;
    const data = getQuery.data.value;
    if (data) {
      processGetData(data, store);
    }
  };
  watch(getQuery.data, (newData) => {
    const data = newData;
    if (data) {
      processGetData(data, store);
    }
  });
  const createQuery = useMutation(
    (body) => createRequest(resourceType, body),
    {
      onSuccess: (json, body) => {
        if (typeof json === "string") {
          const currentData = store.data[body.id || ""];
          const transformedData = removeNullKeys(body, currentData);
          store.data[body.id || ""] = defu(body, transformedData);
        } else {
          const elementData = json.data;
          const transformedData = removeNullKeys(elementData, store.data[elementData.id]);
          store.data[elementData.id] = defu(elementData, transformedData);
        }
      },
      ...queryOptions?.create
    }
  );
  const create = async (body) => {
    createQuery.mutate(body);
  };
  const updateQuery = useMutation(
    (params) => updateRequest(resourceType, params.id, params.body),
    {
      onSuccess: (json, variables) => {
        if (typeof json === "string") {
          store.get(variables.id);
        } else {
          const elementData = json.data;
          const transformedData = removeNullKeys(elementData, store.data[elementData.id]);
          store.data[elementData.id] = defu(elementData, transformedData);
        }
      },
      ...queryOptions?.update
    }
  );
  const update = async (id, body) => {
    updateQuery.mutate({ id, body });
  };
  const deleteQuery = useMutation(
    (id) => deleteRequest(resourceType, id),
    {
      onSuccess: (_, id) => {
        delete store.data[id];
      },
      ...queryOptions?.delete
    }
  );
  const vDelete = async (id) => {
    deleteQuery.mutate(id);
  };
  return {
    index,
    indexQuery,
    get,
    getQuery,
    create,
    createQuery,
    update,
    updateQuery,
    delete: vDelete,
    deleteQuery
  };
};
